# Read an Excel sheet containing Fourier coefficients into a data frame
# and create a JointsAndDrivers file with the corresponding Forurier drivers

from jinja2 import Template

anyman_template = Template(
    """

// Anthropometrics file generated based on ANSUR PCA scaling 
Main.HumanModel.Anthropometrics = 
{
 BodyMass = {{ data.get("Main.HumanModel.Anthropometrics.BodyMass", "75; // Not defined in dataset")}};
 BodyHeight = {{ data.get("Main.HumanModel.Anthropometrics.BodyHeight", "1.8; // Not defined in dataset")}};
};    
    
Main.HumanModel.Anthropometrics.SegmentDimensions = 
{
  {%- for key, val in data.items() %}
  {%- set first, is_anthro, varname = key.partition("Main.HumanModel.Anthropometrics.SegmentDimensions.") -%}
  {%- if is_anthro %}
  {{varname}} = DesignVar({{val}});
  {%- endif -%}
  {%- endfor %}
};
    """
)


def CreateAnyMan(X, anyfile):

    with open(anyfile, "w+") as fh:
        fh.write(anyman_template.render(data=X))
