import wx
import os
import pkg_resources
from configobj import ConfigObj
from ansur_plugin import res

class StaticText(wx.StaticText):
    def __init__(self, parent, label, custom_id):
        wx.StaticText.__init__(self, parent, label=label)
        self.custom_id = custom_id

class TextCtrl(wx.TextCtrl):
    def __init__(self, parent, custom_id):
        wx.TextCtrl.__init__(self, parent, style=wx.TE_PROCESS_ENTER)
        self.custom_id = custom_id

class CheckBox(wx.CheckBox):
    def __init__(self, parent, label, custom_id):
        wx.CheckBox.__init__(self, parent, label=label)
        self.custom_id = custom_id

class Panel(wx.Panel):
    def __init__(self, parent, custom_id):
        wx.CheckBox.__init__(self, parent)
        self.custom_id = custom_id

class AnatomicalDmnsns(wx.Panel):
    def __init__(self, parent, ansur_be):
        wx.Panel.__init__(self, parent=parent)
        wx.ToolTip.Enable(True)
        self.ansur_be = ansur_be
        self.ini_file = ConfigObj(res.anatomical_dmnsns)
        
        v_sizer = wx.BoxSizer(wx.VERTICAL)
        self.frame = parent

        self.text_ctrls = {}
        self.chboxes = {}
        self.panels = {}
        v_p = 3
        panel_v_p = 3
        h_p = 3
        chbox_offset = 100
        text_offset = 25

        #used to find coordinates to place textCtrls
        def on_hover(event):
            name = event.GetEventObject().custom_id
            value = self.ini_file[name]['value']

        def on_chbox(self, event, text_ctrl):
            checked = event.GetEventObject().GetValue()
            parameter = event.GetEventObject().custom_id
            value = text_ctrl.GetValue()
            if checked is True:
                text_ctrl.SetEditable(False)
                ini_file = ConfigObj(res.anatomical_dmnsns)
                ini_file[parameter]['value'] = ini_file[parameter]['default_value']
                text_ctrl.SetValue(str(round(float(ini_file[parameter]['default_value']),2)))
                self.ansur_be.set_auto(parameter, 'True')
                self.text_ctrls[parameter].SetForegroundColour((0,0,0))
                self.text_ctrls[parameter].Refresh()
                self.ansur_be.update_ini()
                self.update_tab()
            else:
                text_ctrl.SetEditable(True)
                text_ctrl.SetValue(value)
                self.ansur_be.set_auto(parameter, 'False')

        def on_textctrl(event):
            text_ctrl = event.GetEventObject()
            parameter = text_ctrl.custom_id
            value=str(text_ctrl.GetValue())
            if value != '' and value != '0.0' and value != '0' and value != '0.' and value != '.0':
                ansur_be.set_value(parameter, value)
                self.update_tab()

        for key in self.ini_file.keys():
            panel = Panel(self, key)
            label = StaticText(panel, label=key + ':  ', custom_id=key)
            label.SetPosition((h_p, v_p))
            label.SetToolTip(wx.ToolTip(self.ansur_be.get_description(key)))
            chbox = CheckBox(panel, label='', custom_id=key)
            chbox.SetPosition((h_p+chbox_offset, v_p))
            chbox.SetValue(True)
            text_ctrl = TextCtrl(panel, custom_id=key)
            text_ctrl.Bind(wx.EVT_ENTER_WINDOW, on_hover)
            text_ctrl.SetPosition((h_p+chbox_offset+text_offset, v_p))
            text_ctrl.SetEditable(False)
            text_ctrl.SetValue(self.ansur_be.get_value(key, "value"))
            text_ctrl.Bind(wx.EVT_TEXT, on_textctrl)
            text_ctrl.Bind(wx.EVT_LEFT_DCLICK, self.on_dclick)
            text_ctrl.SetSize(50,-1)
            self.text_ctrls[key] = text_ctrl
            self.chboxes[key] = chbox
            self.panels[key] = panel
            panel.SetSize((200, 27))
            panel.SetPosition((h_p, panel_v_p))
            panel_v_p = panel_v_p + 30
            chbox.Bind(wx.EVT_CHECKBOX, lambda event, temp=text_ctrl: on_chbox(self, event, temp))   

        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        
        # Currently right and left sides of body are mirrored.
        # TODO: Perhaps add switch for mirrored or side dependent, if side dependent then add additional tabs?
        self.panels['NeckLength'].SetPosition(((340, 5)))
        self.panels['HandBreadth'].SetPosition(((340, 35)))
        self.panels['HandLength'].SetPosition(((340, 65)))

        self.panels['UpperArmLength'].SetPosition(((340, 150)))
        self.panels['ForearmLength'].SetPosition(((h_p, 150)))

        self.panels['TrunkHeight'].SetPosition(((340, 210)))
        self.panels['TrunkWidth'].SetPosition(((340, 240)))

        self.panels['PelvisHeight'].SetPosition(((h_p, 210)))
        self.panels['PelvisDepth'].SetPosition(((h_p, 240)))
        self.panels['PelvisWidth'].SetPosition(((h_p, 270)))


        self.panels['ThighLength'].SetPosition(((h_p, 350)))
        self.panels['ShankLength'].SetPosition(((h_p, 410)))

        self.panels['FootLength'].SetPosition(((340, 430)))
        self.panels['FootWidth'].SetPosition(((340, 460)))
        self.panels['FootHeight'].SetPosition(((340, 490)))

    def OnEraseBackground(self, evt):        
        dc = evt.GetDC()
        if not dc:
            dc = wx.ClientDC(self)
            rect = self.GetUpdateRegion().GetBox()
            dc.SetClippingRect(rect)
        dc.Clear()
        bmp = wx.Bitmap(pkg_resources.resource_filename('ansur_plugin',
                                     "GordonImages/AnatomicalDimensions.png"))
        image = bmp.ConvertToImage()
        image = image.Scale(500, 500, wx.IMAGE_QUALITY_HIGH)
        result = wx.Bitmap(image)
        dc.DrawBitmap(result, 40, 25)

    def on_dclick(self, event):
            text_ctrl = event.GetEventObject()
            parameter = text_ctrl.custom_id
            text_ctrl.SetEditable(True)
            self.ansur_be.set_auto(parameter, 'False')
            self.chboxes[parameter].SetValue(False)

    def update_tab(self):
        for key in self.text_ctrls.keys():
            parameter = self.text_ctrls[key].custom_id
            if not self.text_ctrls[key].IsEditable():
                value = self.ansur_be.get_value(parameter, "value")
                self.text_ctrls[key].ChangeValue(value)
                self.text_ctrls[key].SetForegroundColour((0,0,0))
            else:
                pass
            #LM= lagrange multiplier in INI file
            LM = self.ansur_be.get_value(parameter, "LM")
            if LM != "": 
                if float(LM) >= 3:
                    self.text_ctrls[key].SetForegroundColour((255,0,0))  
                    self.text_ctrls[key].Refresh()    
                else:
                    self.text_ctrls[key].SetForegroundColour((0,0,0))
                    self.text_ctrls[key].Refresh()
        
