from pathlib import Path
from configobj import ConfigObj
from ams_service import Connection
from ansur_plugin import res
import  ansur_plugin.vended.create_subject as create_subject
from ansur_plugin.CreateAnyMan import CreateAnyMan
from ansur_plugin.basicinput import BasicInput
from ansur_plugin.measurementsinput import MeasurementsInput
from ansur_plugin.anatomicadmnsns import AnatomicalDmnsns
import wx
import numpy as np


##TO DO
# add 'average' to gender
# expand statusbar/button to see more details of LM and STDs the certain constraint is away from 'mean'
# link button_conflict popup box to set textctrls to default value when chosen to be reverted by user

class AnsurBE:
    def __init__(self, port, plugin_id, textStatusBar, statusbar, button_conflict):
        self.images = ConfigObj(res.images_address)
        self.parameters = ConfigObj(res.parameters_address)
        self.anatomicaldmnsns = ConfigObj(res.anatomical_dmnsns)
        self.textStatusBar= textStatusBar
        self.statusbar = statusbar
        self.button_conflict = button_conflict

        if port and plugin_id:
            self.connection = Connection(port, plugin_id)
        else:
            self.connection = None
        self.ansur = create_subject.ANSURModel()

        ##**where to find mean and std data for ansur population
        ##use the Xmean values to assign to default_values, this way if the ANSUr database gets changed they will be updated
        print('average body height', self.ansur.model._Xmean["Main.HumanModel.Anthropometrics.BodyHeight"])
        print('stdev body height', self.ansur.model._Xstd["Main.HumanModel.Anthropometrics.BodyHeight"])

        #defines initial average subject (mean of male and female populations)
        self.parameters_dict = {}
        info = self.ansur.CreateSubject_FullInfo(self.parameters_dict)
        self.subject = info["result"]
        self.pcs = info["principal components"]
        self.lms = info["lagrange multipliers"]

    def set_auto(self, parameter, auto):
        ##this chunk of code isn't quite correct
        # for key in self.images.keys():
        #     if parameter == self.images[key]["matrix_name"] or parameter in self.images.keys():
        #         print('image parameter found')
        #         self.images[key]["auto"] = str(auto)
        #         self.images.write()
        #         print(self.images[key]["value"])
        #         return self.images[key]["value"]
        # for key in self.parameters.keys():
        #     if parameter == self.parameters[key]["matrix_name"] or parameter in self.parameters.keys():
        #         print('parameter parameter found')
        #         self.parameters[key]["auto"] = str(auto)
        #         self.parameters.write()
        #         return self.parameters[key]["value"]
        # for key in self.anatomicaldmnsns.keys():
        #     if parameter == self.anatomicaldmnsns[key]["matrix_name"] or parameter in self.anatomicaldmnsns.keys():
        #         print('anatomicaldmnsn parameter found')
        #         self.anatomicaldmnsns[key]["auto"] = str(auto)
        #         self.anatomicaldmnsns.write()
        #         return self.anatomicaldmnsns[key]["value"]

        ##this doesn't find values for self.images, self.parameters when using in self.button_conflict because they are 'matrix_names' and not 'keys'
        print('parameter passed=', parameter)
        if parameter in self.images.keys():
            self.images[parameter]["auto"] = str(auto)
            self.images.write()
            return self.images[parameter]["value"]
        elif parameter in self.parameters.keys():
            self.parameters[parameter]["auto"] = str(auto)
            self.parameters.write()
            return self.parameters[parameter]["value"]
        elif parameter in self.anatomicaldmnsns.keys():
            self.anatomicaldmnsns[parameter]["auto"] = str(auto)
            self.anatomicaldmnsns.write()
            return self.anatomicaldmnsns[parameter]["value"]

    # Python code to merge dict using update() method
    def Merge(self, dict1, dict2):
        return dict2.update(dict1)

    def set_value(self, parameter, value):
        #Only add this parameter to constraint list if male/female,  NOT both
        self.parameters_dict = {"GENDER": float(self.parameters["Gender"]["value"])} 
         
        if parameter in self.images.keys():
            self.parameters_dict[self.images[parameter]["matrix_name"]] = float(value)
            self.images[parameter]["value"] = str(value)
            self.images.write()
        elif parameter in self.parameters.keys():
            self.parameters_dict[self.parameters[parameter]["matrix_name"]] = float(value)
            self.parameters[parameter]["value"] = str(value)
            self.parameters.write()
        elif parameter in self.anatomicaldmnsns.keys():
            self.parameters_dict[self.anatomicaldmnsns[parameter]["matrix_name"]] = float(value)
            self.anatomicaldmnsns[parameter]["value"] = str(value)
            self.anatomicaldmnsns.write()
        self.update_ini()

    def get_value(self, parameter, measure):
        self.images = ConfigObj(res.images_address)
        self.parameters = ConfigObj(res.parameters_address)
        self.anatomicaldmnsns = ConfigObj(res.anatomical_dmnsns)
        if parameter in self.images.keys():
            value = self.images[parameter][measure]
            if value != "":
                value = round(float(value), 2)
        elif parameter in self.parameters.keys():
            value = self.parameters[parameter][measure]
            if value != "":
                value = round(float(value), 2)
        elif parameter in self.anatomicaldmnsns.keys():
            value=self.anatomicaldmnsns[parameter][measure]
            if value != "":
                value = round(float(value), 2)
        return str(value)

    def get_description(self, parameter):
        self.images = ConfigObj(res.images_address)
        self.anatomicaldmnsns = ConfigObj(res.anatomical_dmnsns)
        if parameter in self.images.keys():
            return self.images[parameter]["description"]
        else:
            return self.anatomicaldmnsns[parameter]["description"]

    def update_ini(self):
        #Only add this parameter to constraint list if male/female,  NOT both
        self.parameters_dict = {"GENDER": float(self.parameters["Gender"]["value"])}

        for entry in self.images:
            if self.images[entry]["auto"] == "False":
                value = self.images[entry]["value"]
                key = self.images[entry]["matrix_name"]
                self.parameters_dict[key] = float(value)

        for entry in self.parameters:
            if self.parameters[entry]["auto"] == "False":
                value = self.parameters[entry]["value"]
                key = self.parameters[entry]["matrix_name"]
                self.parameters_dict[key] = float(value)

        for entry in self.anatomicaldmnsns:
            if self.anatomicaldmnsns[entry]["auto"] == "False":
                value = self.anatomicaldmnsns[entry]["value"]
                key = self.anatomicaldmnsns[entry]["matrix_name"]
                self.parameters_dict[key] = float(value)

        
        info = self.ansur.CreateSubject_FullInfo(self.parameters_dict)
        
        self.subject = info["result"]
        self.pcs = info["principal components"]
        self.lms = info["lagrange multipliers"]

        #construct empty ndarray containing constraints, user assigned values, and lagranian multipliers to be used for outlier check
        self.constraints= np.full([len(self.parameters_dict),3], None)
        # self.constraints= [[None]*len(self.parameters_dict)]*3
        i=0
        for key in self.parameters_dict:
            self.constraints[i][0]=key
            self.constraints[i][1]=self.parameters_dict[key]
            self.constraints[i][2]=self.lms[i] if len(self.lms)>i else 0
            i=i+1

        conflict= False
        for lm in self.constraints[:,[2]]: 
            if round(abs(float(lm)),2) >= 4:
                conflict=True
                break
            
        if conflict==True:
            self.textStatusBar.SetForegroundColour((255,0,0))
            self.textStatusBar.SetLabelText('*Red value(s) beyond 4 stdev')
            self.statusbar.SetStatusText('Conflicting constraints exist: ', 1)
            # self.button_conflict.Show()
            self.button_conflict.Bind(wx.EVT_BUTTON, self.onButton)
        else: 
            self.textStatusBar.SetForegroundColour((0,0,0))
            self.textStatusBar.SetLabelText('No conflict constraints')
            self.statusbar.SetStatusText('', 1)
            self.button_conflict.Hide()

        for entry in self.images.keys():
            name = self.images[entry]["matrix_name"]
            if self.images[entry]["auto"] == "True":
                try:
                    self.images[entry]["value"] = self.subject[name]
                except:
                    pass
            # check to see if entry is a constraint, if so add Lagrange multiplier to INI file
            result=np.where(self.constraints==name)
            idx=result[0]
            if idx != None:
                self.images[entry]["LM"] = abs(round(float(self.constraints[int(idx)][2])))
            else:
                self.images[entry]["LM"] = ''

        for entry in self.parameters:
            name = self.parameters[entry]["matrix_name"]
            if name != "" and self.parameters[entry]["auto"] == "True":
                try:
                    self.parameters[entry]["value"] = self.subject[name]
                except:
                    pass
            # check to see if entry is a constraint, if so add Lagrange multiplier to INI file
            constraint=np.where(self.constraints==name)
            idx=constraint[0]
            if idx != None:
                self.parameters[entry]["LM"] = abs(round(float(self.constraints[int(idx)][2]),2))
            else:
                self.parameters[entry]["LM"] = ''

        for entry in self.anatomicaldmnsns:
            name = self.anatomicaldmnsns[entry]["matrix_name"]
            if name != "" and self.anatomicaldmnsns[entry]["auto"] == "True":
                try:
                    self.anatomicaldmnsns[entry]["value"] = self.subject[name]
                except:
                    pass
            # check to see if entry is a constraint, if so add Lagrange multiplier to INI file
            constraint=np.where(self.constraints==name)
            idx=constraint[0]
            if idx != None:
                self.anatomicaldmnsns[entry]["LM"] = abs(round(float(self.constraints[int(idx)][2]),2))
            else:
                self.anatomicaldmnsns[entry]["LM"] = ''

        self.images.write()
        self.parameters.write()
        self.anatomicaldmnsns.write()

    def create_any_man(self):
        try:
            anyfile = Path(self.connection.get_paths()["ANSUR_PLUGIN_ANYMAN_FILE"])
        except KeyError:
            raise ("Please define '#path ANSUR_PLUGIN_ANYMAN_FILE' statement which the plugin will write to")
        CreateAnyMan(self.subject, anyfile)

    def load_model(self):
        self.connection.load_model()

    def default(self):
        for entry in self.images:
            self.images[entry]["auto"] = "True"
        self.images.write()

        for entry in self.parameters:
            if entry != "Gender":
                self.parameters[entry]["auto"] = "True"
        self.parameters.write()

        for entry in self.anatomicaldmnsns:
            self.anatomicaldmnsns[entry]["auto"] = "True"
        self.anatomicaldmnsns.write()

        self.update_ini()

    def onButton(self, event):
        ##Need to go back and exclude Gender from the list in pop up box, might be odd for user to see LM based on gender constraint
        keys=list(self.parameters_dict.keys())
        values=list(self.parameters_dict.values())
        LMs=abs(self.lms)
        list_info=[None]*len(keys)
        for x in range(len(keys)):
            if keys[x].startswith('Main.HumanModel.Anthropometrics.SegmentDimensions.'):
                keys[x]=keys[x][50:]
            list_info[x]= f"{keys[x]}:   assigned value ({values[x]}) is ({round(LMs[x])}) stdevs beyond population mean"
        dlg = wx.MultiChoiceDialog(None, "Select which constraints you'd like to restore to auto set values",
                                   "Warning: Unrealistic Anthropometry", list_info)
        
        if (dlg.ShowModal() == wx.ID_OK):
            selections = dlg.GetSelections()
            for y in selections:
                parameter= keys[y]
                self.set_auto(parameter, True)
                self.update_ini()
                ##need to call the tabs and 'updatetab' in order to have chbox appear/value go to default/color change
                #Like how it is done in frame.py tab but need to define/link these classes.
                # basic_input.update_tab()
                # measurmentsinput.update_tab()
                # anatomicaldmnsns.update_tab()
        
        dlg.Destroy()
