import wx
import os
import pkg_resources
from configobj import ConfigObj
from ansur_plugin import res

#todo: 
# when using on_chbox for FatPercent, rounding doesnt occur
# Find out how to change width of textctrl within sizer
# center image and boxes when frame expands

class StaticText(wx.StaticText):
    def __init__(self, parent, label, custom_id):
        wx.StaticText.__init__(self, parent, label=label)
        self.custom_id = custom_id

class TextCtrl(wx.TextCtrl):
    def __init__(self, parent, custom_id):
        wx.TextCtrl.__init__(self, parent, style=wx.TE_PROCESS_ENTER)
        self.custom_id = custom_id

class CheckBox(wx.CheckBox):
    def __init__(self, parent, label, custom_id):
        wx.CheckBox.__init__(self, parent, label=label)
        self.custom_id = custom_id

class BasicInput(wx.Panel):
    def __init__(self, parent, ansur_be):
        wx.Panel.__init__(self, parent)
        self.ansur_be = ansur_be
        self.ini_file = ConfigObj(res.parameters_address)
        sizer_grid = wx.FlexGridSizer(rows=11, cols=1,vgap=5, hgap=5)
        self.frame = parent
        self.text_ctrls = {}
        self.chboxes = {}

        def on_textctrl(event):
            text_ctrl = event.GetEventObject()
            parameter = text_ctrl.custom_id
            value=str(text_ctrl.GetValue())
            if value != '' and value != '0.0' and value != '0' and value != '0.' and value != '.0':
                ansur_be.set_value(parameter, value)
                self.update_tab()
                
        def choice(event):
            gender = event.GetEventObject().GetSelection()
            # need to add one because Compile.csv is on [1,2] scale for gender and choicebox is [0,1]
            gender=int(gender)+1
            ansur_be.set_value(parameter, str(gender))
            if gender == 3:
                ansur_be.set_auto(parameter, True)
            #Switch to add parameter to constraint list (True:not a constraint, False: constraint)
            else:
                ansur_be.set_auto(parameter, False)
            self.update_tab()

        for key in self.ini_file.keys():
            if key != 'Gender':
                sizer_h =wx.FlexGridSizer(rows=1, cols=3,vgap=3, hgap=3)
                label = StaticText(self, label=key + ':  ', custom_id=key)
                chbox = CheckBox(self, 'Auto Set Value', custom_id=key)
                chbox.SetValue(True)
                text_ctrl = TextCtrl(self, custom_id=key)
                text_ctrl.SetEditable(False)
                text_ctrl.SetSize(50,-1)
                parameter = text_ctrl.custom_id
                text_ctrl.SetValue(str(round(float(self.ini_file[parameter]['default_value']),2)))
                text_ctrl.Bind(wx.EVT_TEXT, on_textctrl)
                text_ctrl.Bind(wx.EVT_LEFT_DCLICK, self.on_dclick)
                self.text_ctrls[key] = text_ctrl
                self.chboxes[key] = chbox
                chbox.Bind(wx.EVT_CHECKBOX, lambda event, temp=text_ctrl: self.on_chbox(event, temp))
                sizer_h.Add(text_ctrl)
                sizer_h.Add(chbox)
                sizer_h.AddGrowableCol(0,2)
            else:  
                parameter='Gender'
                sizer_h =wx.FlexGridSizer(rows=1, cols=2,vgap=3, hgap=3)
                label = StaticText(self, label= f'Sample Population ({key}):  ', custom_id=key)
                # text_ctrl = wx.ComboBox(self, choices=['Male', 'Female', 'Both'])
                text_ctrl = wx.ComboBox(self, choices=['Male', 'Female'])
                text_ctrl.Bind(wx.EVT_COMBOBOX, choice)
                # 'default_values' assigned each time app starts up, should change this to assign directly from compiled data Xmeans
                # need to subtract 1 because ComboBox is on [0,1] scale for gender
                text_ctrl.SetSelection(int(self.ini_file[parameter]['default_value'])-1)
                ansur_be.set_value(parameter, self.ini_file[parameter]['default_value'])
                sizer_h.Add(text_ctrl)
                sizer_h.AddGrowableCol(0,1) 

            sizer_grid.Add(label,1)
            sizer_grid.Add(sizer_h,1)
        
        sizer_grid.AddGrowableRow(10,1)
        self.SetSizer(sizer_grid)
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
    
    def on_dclick(self, event):
        text_ctrl = event.GetEventObject()
        parameter = text_ctrl.custom_id
        text_ctrl.SetEditable(True)
        self.ansur_be.set_auto(parameter, False)
        self.chboxes[parameter].SetValue(False)

    def OnEraseBackground(self, evt):        
        dc = evt.GetDC()
        if not dc:
            dc = wx.ClientDC(self)
            rect = self.GetUpdateRegion().GetBox()
            dc.SetClippingRect(rect)
        dc.Clear()
        bmp = wx.Bitmap(pkg_resources.resource_filename('ansur_plugin', "GordonImages/BasicInput.png"))

        image = bmp.ConvertToImage()
        image = image.Scale(500, 500, wx.IMAGE_QUALITY_HIGH)
        result = wx.Bitmap(image)
        dc.DrawBitmap(result, 40, 25)

    def update_tab(self):
        for key in self.text_ctrls.keys():
            if key != 'Gender':
                parameter = self.text_ctrls[key].custom_id
                if not self.text_ctrls[key].IsEditable():
                    value = self.ansur_be.get_value(parameter, "value")
                    self.text_ctrls[key].ChangeValue(value)
                    self.text_ctrls[key].SetForegroundColour((0,0,0))
                    self.text_ctrls[key].Refresh()
                else: 
                    pass
                #LM= lagrange multiplier in INI file
                LM = self.ansur_be.get_value(parameter, "LM")
                if LM != "": 
                    if float(LM) >= 4:
                        self.text_ctrls[key].SetForegroundColour((255,0,0))
                        self.text_ctrls[key].Refresh()      
                    else:
                        self.text_ctrls[key].SetForegroundColour((0,0,0))
                        self.text_ctrls[key].Refresh()
            else:
                pass

    def on_chbox(self, event, text_ctrl):
        self.Freeze()
        checked = event.GetEventObject().GetValue()
        parameter = event.GetEventObject().custom_id
        value = text_ctrl.GetValue()
        if checked is True:
            text_ctrl.SetEditable(False)
            self.ini_file[parameter]['value'] = self.ini_file[parameter]['default_value']
            text_ctrl.SetValue(str(round(float(self.ini_file[parameter]['default_value']),2)))
            self.ansur_be.set_auto(parameter, True)
            text_ctrl.SetForegroundColour((0,0,0))
            self.ansur_be.update_ini()
            self.update_tab()
        else:
            text_ctrl.SetEditable(True)
            text_ctrl.SetValue(value)
            self.ansur_be.set_auto(parameter, False)
        self.Thaw()
    
    def onload(self, connection):
        self.Parent.Parent.Parent.Hide()
        connection.load_model()
        self.Parent.Parent.Parent.Close()