import sys

import wx
app = wx.App()

from ansur_plugin.ansur import AnsurBE
from ansur_plugin.basicinput import BasicInput
from ansur_plugin.measurementsinput import MeasurementsInput
from ansur_plugin.anatomicadmnsns import AnatomicalDmnsns
from ansur_plugin.abouttab import AboutTab
from ansur_plugin import res
import numpy as np

class Frame(wx.Frame):
    def __init__(self, parent, title, args):
        super(Frame, self).__init__(parent, title=title,
                                    style=(wx.DEFAULT_FRAME_STYLE ^
                                           wx.RESIZE_BORDER))
        #create status bar/button for conflicting constraints, hid until triggered                                   
        self.statusbar=self.CreateStatusBar(2)
        self.statusbar.SetStatusText('', 1)
        self.textStatusBar = wx.StaticText(self.statusbar, -1)
        self.textStatusBar.SetLabelText('')
        self.textStatusBar.SetBackgroundColour(None)
        self.textStatusBar.SetPosition((2, 5))
        self.button_conflict=wx.Button(self.statusbar, -1)
        self.button_conflict.SetLabelText('Click to resolve')
        w1= self.statusbar.Size[0]
        w1=w1+110
        self.button_conflict.SetPosition((w1, 0))
        self.button_conflict.Hide()

        self.ansur_be = AnsurBE(args['port'], args['identification'], self.textStatusBar, self.statusbar, self.button_conflict)
        self.ansur_be.default()
        self.SetIcon(wx.Icon(res.icon , wx.BITMAP_TYPE_ICO))
        frame_sizer = wx.BoxSizer(wx.VERTICAL)
        panel = wx.Panel(self)

        sizer = wx.BoxSizer(wx.VERTICAL)

        tabs = wx.Notebook(panel)
        basic_input = BasicInput(tabs, self.ansur_be)
        measurmentsinput = MeasurementsInput(tabs, self.ansur_be)
        anatomicaldmnsns = AnatomicalDmnsns(tabs, self.ansur_be)
        about_tab = AboutTab(tabs)
        tabs.AddPage(basic_input, 'Basic Input')
        tabs.AddPage(measurmentsinput, 'Functional Dimensions')
        tabs.AddPage(anatomicaldmnsns, 'Anatomical Dimensions')
        tabs.AddPage(about_tab, 'About')

        def update(event):
            basic_input.update_tab()
            measurmentsinput.update_tab()
            anatomicaldmnsns.update_tab()

        tabs.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, update)

        sizer.Add(tabs, 1, wx.ALL | wx.EXPAND, 3)

        buttons_sizer = wx.BoxSizer(wx.HORIZONTAL)
        label_button = wx.StaticText(panel,
                                     label='Load the ANSUR AnyMan parameters:')

        buttons_sizer.Add(label_button, 0, wx.TOP, 6)

        def onload(event): self.onload(self.ansur_be)
        
        button = wx.Button(panel, label='Load to AMS')
        button.Bind(wx.EVT_BUTTON, onload)

        self.Bind(wx.EVT_CLOSE, self.on_exit)

        if self.ansur_be.connection is None or not self.ansur_be.connection.is_connected():
            button.Disable()

        buttons_sizer.Add(button, 0, wx.ALL, 3)
        sizer.Add(buttons_sizer, 0, wx.ALL, 3)
        panel.SetSizer(sizer)
        frame_sizer.Add(panel, 1, wx.EXPAND)
        self.SetSizer(frame_sizer)

        self.SetSize((600, 650))
        self.Show()

    def onload(self, ansur_be):
        ansur_be.create_any_man()
        ansur_be.load_model()
        print('onload')

    def on_exit(self, event):
        self.Destroy()
        sys.exit(0)