import wx
import os
from configobj import ConfigObj
from ansur_plugin import res

## Functional dimensions tab
# Add some warning if value == 0 or "" when switching tabs or to next/previous measurement:
# wx.MessageDialog 'did you leave a functional dimension blank?'

class ImageControl(wx.StaticBitmap):
    def __init__(self, parent, ansur_be, image=None):
        wx.StaticBitmap.__init__(self, parent)
        h_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.ansur_be = ansur_be
        self.im = ConfigObj(res.images_address).keys()[image]
        image = ConfigObj(res.images_address)[self.im]
        full_address = os.path.join(res.gordon_images_address, image['address'])
        img = wx.Image(full_address, wx.BITMAP_TYPE_ANY)
        self.SetBitmap(wx.Bitmap(img))
        self.parent = parent

        def on_text(event): 
            self.on_text(event, self.im)
        
        def on_dclick(event): 
            text_ctrl = event.GetEventObject()
            self.on_dclick(text_ctrl, self.im)
        
        self.text = wx.TextCtrl(self,style=wx.TE_PROCESS_ENTER)
        self.text.SetValue(str(round(float(image['value']),2)))
        self.text.SetSize(50,-1)
        self.text.Bind(wx.EVT_TEXT, on_text)
        self.text.Bind(wx.EVT_LEFT_DCLICK, on_dclick)
        self.label = wx.StaticText(self, label=image['label'])
        self.label.SetSize(200,-1)
        def on_chbox(event): self.on_chbox(event, self.im)
        self.chbox = wx.CheckBox(self, label="Auto Set Value")
        self.chbox.Bind(wx.EVT_CHECKBOX, on_chbox)
        chbox = bool(image['auto'])
        self.chbox.SetValue(chbox)
        if chbox:
            self.text.SetEditable(False)
        else:
            self.text.SetEditable(True)
        
        h_sizer.Add(self.chbox, 1 , wx.ALIGN_LEFT, 0)
        h_sizer.Add(self.text, 1 , wx.ALIGN_LEFT, 0)
        h_sizer.Add(self.label, 1 , wx.RIGHT, 10)

        self.SetSizer(h_sizer)
        
    def on_dclick(self, text_ctrl, image):
        self.text.SetEditable(True)
        self.ansur_be.set_auto(image, 'False')
        self.chbox.SetValue(False)

    def on_chbox(self, event, image):
        checked = event.GetEventObject().GetValue()
        value = self.text.GetValue()
        if checked is True:
            self.text.SetEditable(False)
            
            ini_file = ConfigObj(res.images_address)
            ini_file[image]['value'] = ini_file[image]['default_value']
            ini_file.write()
            self.text.SetValue(str(round(float(ini_file[image]['default_value']),2)))
            self.ansur_be.set_auto(image, 'True')
            self.text.SetSize(50,-1) 
            self.text.SetForegroundColour((0,0,0))
            self.ansur_be.update_ini()
            self.parent.update_tab()
        else:
            self.text.SetEditable(True)
            self.text.SetValue(value)
            self.ansur_be.set_auto(image, 'False')

    def on_text(self, event, image):
        value = event.GetEventObject().GetValue()
        if value != '' and value != '0.0' and value != '0' and value != '0.' and value != '.0':
            self.ansur_be.set_value(image, value)
            #LM= lagrange multiplier in INI file
            LM = self.ansur_be.get_value(self.im , "LM")
            if LM != "": 
                if float(LM) >= 4:
                    self.text.SetSize(50,-1) 
                    self.text.SetForegroundColour((255,0,0))  
                    self.text.Refresh()    
                else:
                    self.text.SetForegroundColour((0,0,0))
                    self.text.Refresh() 
        else:
            pass
        

    def set_image(self, image=None):
        self.im = image
        image = ConfigObj(res.images_address)[image]
        self.name = image['name']
        self.SetBitmap(wx.Bitmap.FromRGBA(400, 400, 255, 255, 255, 255))
        full_adress = os.path.join(res.gordon_images_address, image['address'])
        img = wx.Image(full_adress, wx.BITMAP_TYPE_ANY)
        self.SetBitmap(wx.Bitmap(img))
        self.text.ChangeValue(str(round(float(image['value']),2)))
        self.text.SetSize(50,-1)
        self.label.SetLabel(image['label'])
        self.label.SetToolTip(wx.ToolTip(image['description']))
        chbox = image['auto'] == 'True'
        self.chbox.SetValue(chbox)
        if chbox:
            self.text.SetEditable(False)
            self.text.SetForegroundColour((0,0,0))
            self.text.Refresh()
        else:
            self.text.SetEditable(True)

class MeasurementsInput(wx.Panel):
    def __init__(self, parent, ansur_be):
        wx.Panel.__init__(self, parent)
        self.ansur_be = ansur_be
        sizer = wx.BoxSizer(wx.VERTICAL)
        self.current_image = 0
        ini_file = ConfigObj(res.images_address)
        self.images_list = ini_file.keys()
        self.values_list = []
        choices = []

        for image in self.images_list:
            choices.append(ini_file[image]['name'])
        self.imageCtrl = ImageControl(self, ansur_be, image=0)
        self.next_button = wx.Button(self, label="Next")
        self.next_button.Bind(wx.EVT_BUTTON, self.next)
        self.previous_button = wx.Button(self, label="Previous")
        self.previous_button.Disable()
        self.previous_button.Bind(wx.EVT_BUTTON, self.previous)
        self.menu = wx.ComboBox(self, choices = choices)
        self.menu.Bind(wx.EVT_COMBOBOX, self.choice)
        self.menu.SetSelection(0)
        sizer_menu = wx.BoxSizer(wx.HORIZONTAL)
        sizer_menu.Add(self.previous_button, 1, wx.ALL, 3)
        sizer_menu.Add(self.menu, 1, wx.ALL, 3)
        sizer_menu.Add(self.next_button, 1, wx.ALL, 3)
        sizer.Add(self.imageCtrl, 1, wx.TOP | wx.ALIGN_CENTER, 25)
        sizer.Add(sizer_menu, 1,  wx.ALIGN_CENTER | wx.ALL, 3)
        self.SetSizer(sizer)
        self.Layout()
        
    def update_tab(self):
        self.Freeze()
        self.imageCtrl.set_image(self.images_list[self.current_image])
        self.Thaw()

    def choice(self, evt):
        self.Freeze()
        self.current_image = evt.GetEventObject().GetSelection()
        self.imageCtrl.set_image(self.images_list[self.current_image])
        if not self.current_image:
            self.previous_button.Disable()
        else:
            self.previous_button.Enable()
        if self.current_image == len(self.images_list)-1:
            self.next_button.Disable()
        else:
            self.next_button.Enable()
        self.Thaw()

    def previous(self, evt):
        self.Freeze()
        self.current_image -= 1
        self.imageCtrl.set_image(self.images_list[self.current_image])
        self.next_button.Enable()
        if not self.current_image:
            evt.GetEventObject().Disable()
        self.menu.SetSelection(self.current_image)
        self.Thaw()

    def next(self, evt):
        self.Freeze()
        self.current_image += 1
        self.imageCtrl.set_image(self.images_list[self.current_image])
        self.previous_button.Enable()
        if self.current_image == len(self.images_list)-1:
            evt.GetEventObject().Disable()
        self.menu.SetSelection(self.current_image)
        self.Thaw()